import 'reflect-metadata';
import { IContainer, Token, Type, Providers, IContainerBuilder, Registration } from '@ts-ioc/core';
import { IModuleBuilder, ModuleEnv, Runnable } from './IModuleBuilder';
import { ModuleConfigure, ModuleConfig } from './ModuleConfigure';
import { LoadedModule } from './ModuleType';
import { IAnnotationBuilder, IAnyTypeBuilder } from './IAnnotationBuilder';
import { ContainerPool } from './ContainerPool';
import { IService } from './Service';
import { IRunner } from './IRunner';
import { IMetaAccessor } from './MetaAccessor';
/**
 * inject module load token.
 *
 * @export
 * @class InjectModuleLoadToken
 * @extends {Registration<T>}
 * @template T
 */
export declare class InjectModuleLoadToken<T> extends Registration<T> {
    constructor(token: Token<T>);
    static classAnnations: any;
}
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
export declare class ModuleBuilder<T> implements IModuleBuilder<T> {
    constructor();
    protected pools: ContainerPool;
    getPools(): ContainerPool;
    setPools(pools: ContainerPool): void;
    protected regDefaultContainer(): IContainer;
    /**
     * get container of the module.
     *
     * @param {(ModuleType | ModuleConfigure)} token module type or module configuration.
     * @param {ModuleEnv} [env] set loadedModule will return loaded container; set default container or not. not set will create new container.
     * @param {IContainer} [parent] set the container parent, default will set root default container.
     * @returns {IContainer}
     * @memberof ModuleBuilder
     */
    getContainer(token: Token<T> | ModuleConfigure, env?: ModuleEnv, parent?: IContainer): IContainer;
    protected getContainerInEnv(env?: ModuleEnv): IContainer;
    protected getConfigId(cfg: ModuleConfigure): string;
    protected setParent(container: IContainer, parent?: IContainer): void;
    createContainer(): IContainer;
    protected containerBuilder: IContainerBuilder;
    getContainerBuilder(): IContainerBuilder;
    protected createContainerBuilder(): IContainerBuilder;
    load(token: Token<T> | ModuleConfigure, env?: ModuleEnv, parent?: IContainer): Promise<LoadedModule>;
    protected cleanLoadModule(container: IContainer, tk: Token<any>): void;
    /**
     * build module.
     *
     * @param {(Token<T> | ModuleConfig<T>)} token
     * @param {ModuleEnv} [env]
     * @param {*} [data]
     * @returns {Promise<T>}
     * @memberof ModuleBuilder
     */
    build(token: Token<T> | ModuleConfig<T>, env?: ModuleEnv, data?: any): Promise<T>;
    /**
    * bootstrap module's main.
    *
    * @param {(Token<T> | ModuleConfig<T>)} token
    * @param {ModuleEnv} [env]
    * @param {*} [data]
    * @returns {Promise<MdInstance<T>>}
    * @memberof ModuleBuilder
    */
    bootstrap(token: Token<T> | ModuleConfig<T>, env?: ModuleEnv, data?: any): Promise<Runnable<T>>;
    getBuilder(token: Token<T> | ModuleConfig<T>, env?: ModuleEnv): IModuleBuilder<T>;
    protected getDefaultBuilder(container: IContainer): IModuleBuilder<any>;
    protected autoRun(container: IContainer, token: Token<any>, cfg: ModuleConfigure, instance: any): Promise<Runnable<T>>;
    protected getDefaultRunner(container: IContainer, ...providers: Providers[]): IRunner<T>;
    protected getDefaultService(container: IContainer, ...providers: Providers[]): IService<T>;
    protected getAnnoBuilder(container: IContainer, token: Token<any>, annBuilder: Token<IAnnotationBuilder<any>> | IAnnotationBuilder<any>): IAnyTypeBuilder;
    protected getDefaultAnnBuilder(container: IContainer): IAnnotationBuilder<any>;
    importModule(token: Token<T> | ModuleConfigure, container: IContainer): Promise<IContainer>;
    getDecorator(): string;
    /**
     * get configuration.
     *
     * @returns {ModuleConfigure}
     * @memberof ModuleBuilder
     */
    getConfigure(token?: Token<T> | ModuleConfigure, container?: IContainer): ModuleConfigure;
    registerDepdences(container: IContainer, config: ModuleConfigure): Promise<ModuleConfigure>;
    /**
     * get module type
     *
     * @protected
     * @param {ModuleConfigure} cfg
     * @returns {Token<T>}
     * @memberof ModuleBuilder
     */
    protected getType(cfg: ModuleConfigure): Token<T>;
    /**
     * get boot type.
     *
     * @protected
     * @param {ModuleConfigure} cfg
     * @returns {Token<T>}
     * @memberof ModuleBuilder
     */
    protected getBootType(cfg: ModuleConfigure): Token<T>;
    protected importConfigExports(container: IContainer, providerContainer: IContainer, cfg: ModuleConfigure): Promise<IContainer>;
    protected registerConfgureDepds(container: IContainer, config: ModuleConfigure): Promise<ModuleConfigure>;
    protected getMetaConfig(token: Token<any>, container: IContainer): ModuleConfigure;
    getDefaultMetaAccessor(container: IContainer, ...providers: Providers[]): IMetaAccessor<any>;
    protected isIocExt(token: Type<any>): boolean;
    protected isDIModule(token: Token<any>): boolean;
    protected registerExts(container: IContainer, config: ModuleConfigure): Promise<IContainer>;
    protected bindProvider(container: IContainer, providers: Providers[]): Token<any>[];
    static classAnnations: any;
}
